package Deque.Application.CapitalGain;

import Deque.*;
import Deque.Implementation.DoublyLinked.*;

/**
 * A class that represents a portfolio of stocks. 
 * By recording their purchase and sale, the
 * captial gains and losses are computed.
 * 
 * @author Frank M. Carrano
 * @version 2.0
 */
public class StockLedger
{
	private DequeInterface<StockPurchase> ledger;
	
	public StockLedger()
	{
		ledger = new LinkedDeque<StockPurchase>();
	} // end default constructor
	
	/*
	 * Task: Records a stock purchase in the ledger.
	*@param sharesBought the number of shares purchased
	*@param pricePerShare the price per share */ 
	public void buy(int sharesBought, double pricePerShare) // 23.17
	{
		StockPurchase purchase = new StockPurchase(sharesBought, pricePerShare);
		ledger.addToBack(purchase);
	} // end buy
	
	/*
	 * Task: Removes sold shares from the ledger and computes the
	 * capital gain or loss.
	*@param sharesSold the number of shares sold
	*@param pricePerShare the price per share
	 * @return the capital gain (loss) */ 
	public double sell(int sharesSold, double pricePerShare) // 23.17
	{
	  double saleAmount = sharesSold * pricePerShare;
	  double totalCost = 0;
	  
	  while (sharesSold > 0)
	  {
	    StockPurchase transaction = ledger.removeFront();
	    double shareCost = transaction.getCostPerShare();
	    int numberOfShares = transaction.getNumberOfShares();
	    
	    if (numberOfShares > sharesSold)
	    {
	      totalCost = totalCost + sharesSold * shareCost;
	      int numberToPutBack = numberOfShares - sharesSold;
	      StockPurchase leftOver = new StockPurchase(numberToPutBack, 
	                                                 shareCost);
	      ledger.addToFront(leftOver); // return leftover shares
	      // Note: loop will exit since sharesSold will be <= 0 later
	    }
	    else
	      totalCost = totalCost + numberOfShares * shareCost;
	    sharesSold = sharesSold - numberOfShares;
	  } // end while
	  
	  return saleAmount - totalCost; // gain or loss
	} // end sell
} // end StockLedger